@@ -22,6 +22,7 @@ import os
 import pexpect
 import subprocess
 import sys
+import time
 import unittest
 
 from UEFI.Filesystems import GrubShellBootableIsoImage
@@ -33,12 +34,13 @@ DPKG_ARCH = subprocess.check_output(
     ['dpkg', '--print-architecture']
 ).decode().rstrip()
 
-
 EfiArchToGrubArch = {
     'X64': "x86_64",
     'AA64': "arm64",
 }
 
+TEST_TIMEOUT = 120
+
 
 def get_local_grub_path(efi_arch, signed=False):
     grub_subdir = "%s-efi" % EfiArchToGrubArch[efi_arch.upper()]
@@ -70,11 +72,18 @@ def get_local_shim_path(efi_arch, signed=False):
 class BootToShellTest(unittest.TestCase):
     debug = True
 
+    def setUp(self):
+        self.startTime = time.time()
+
+    def tearDown(self):
+        t = time.time() - self.startTime
+        sys.stdout.write("%s runtime: %.3fs\n" % (self.id(), t))
+
     def run_cmd_check_shell(self, cmd):
-        child = pexpect.spawn(' '.join(cmd))
+        child = pexpect.spawn(' '.join(cmd), encoding='UTF-8')
 
         if self.debug:
-            child.logfile = sys.stdout.buffer
+            child.logfile = sys.stdout
         try:
             while True:
                 i = child.expect(
@@ -82,7 +91,7 @@ class BootToShellTest(unittest.TestCase):
                         'Press .* or any other key to continue',
                         'Shell> '
                     ],
-                    timeout=60,
+                    timeout=TEST_TIMEOUT,
                 )
                 if i == 0:
                     child.sendline('\x1b')
@@ -91,7 +100,9 @@ class BootToShellTest(unittest.TestCase):
                     child.sendline('reset -s\r')
                     continue
         except pexpect.EOF:
-            return
+            child.close()
+            if child.exitstatus != 0:
+                self.fail("ERROR: exit code %d\n" % (child.exitstatus))
         except pexpect.TIMEOUT as err:
             self.fail("%s\n" % (err))
 
@@ -100,10 +111,10 @@ class BootToShellTest(unittest.TestCase):
             PRE_EXEC = 1
             POST_EXEC = 2
 
-        child = pexpect.spawn(' '.join(cmd))
+        child = pexpect.spawn(' '.join(cmd), encoding='UTF-8')
 
         if self.debug:
-            child.logfile = sys.stdout.buffer
+            child.logfile = sys.stdout
         try:
             state = State.PRE_EXEC
             while True:
@@ -115,7 +126,7 @@ class BootToShellTest(unittest.TestCase):
                         'grub> ',
                         'Command Error Status: Access Denied',
                     ],
-                    timeout=60,
+                    timeout=TEST_TIMEOUT,
                 )
                 if i == 0:
                     child.sendline('\x1b')
@@ -137,10 +148,12 @@ class BootToShellTest(unittest.TestCase):
                 if i == 4:
                     verified = False
                     continue
+        except pexpect.EOF:
+            child.close()
+            if child.exitstatus != 0:
+                self.fail("ERROR: exit code %d\n" % (child.exitstatus))
         except pexpect.TIMEOUT as err:
             self.fail("%s\n" % (err))
-        except pexpect.EOF:
-            pass
         self.assertEqual(should_verify, verified)
 
     def test_aavmf(self):
@@ -188,60 +201,6 @@ class BootToShellTest(unittest.TestCase):
         q = Qemu.QemuCommand(QemuEfiMachine.AAVMF32)
         self.run_cmd_check_shell(q.command)
 
-    def test_ovmf_pc(self):
-        q = Qemu.QemuCommand(
-            QemuEfiMachine.OVMF_PC, flash_size=QemuEfiFlashSize.SIZE_2MB,
-        )
-        self.run_cmd_check_shell(q.command)
-
-    def test_ovmf_q35(self):
-        q = Qemu.QemuCommand(
-            QemuEfiMachine.OVMF_Q35, flash_size=QemuEfiFlashSize.SIZE_2MB,
-        )
-        self.run_cmd_check_shell(q.command)
-
-    def test_ovmf_secboot(self):
-        q = Qemu.QemuCommand(
-            QemuEfiMachine.OVMF_Q35,
-            variant=QemuEfiVariant.SECBOOT,
-            flash_size=QemuEfiFlashSize.SIZE_2MB,
-        )
-        self.run_cmd_check_shell(q.command)
-
-    def test_ovmf_ms(self):
-        q = Qemu.QemuCommand(
-            QemuEfiMachine.OVMF_Q35,
-            variant=QemuEfiVariant.MS,
-            flash_size=QemuEfiFlashSize.SIZE_2MB,
-        )
-        self.run_cmd_check_shell(q.command)
-
-    @unittest.skipUnless(DPKG_ARCH == 'amd64', "amd64-only")
-    def test_ovmf_ms_secure_boot_signed(self):
-        q = Qemu.QemuCommand(
-            QemuEfiMachine.OVMF_Q35,
-            variant=QemuEfiVariant.MS,
-            flash_size=QemuEfiFlashSize.SIZE_2MB,
-        )
-        grub = get_local_grub_path('X64', signed=True)
-        shim = get_local_shim_path('X64', signed=True)
-        iso = GrubShellBootableIsoImage('X64', shim, grub)
-        q.add_disk(iso.path)
-        self.run_cmd_check_secure_boot(q.command, 'x64', True)
-
-    @unittest.skipUnless(DPKG_ARCH == 'amd64', "amd64-only")
-    def test_ovmf_ms_secure_boot_unsigned(self):
-        q = Qemu.QemuCommand(
-            QemuEfiMachine.OVMF_Q35,
-            variant=QemuEfiVariant.MS,
-            flash_size=QemuEfiFlashSize.SIZE_2MB,
-        )
-        grub = get_local_grub_path('X64', signed=False)
-        shim = get_local_shim_path('X64', signed=False)
-        iso = GrubShellBootableIsoImage('X64', shim, grub)
-        q.add_disk(iso.path)
-        self.run_cmd_check_secure_boot(q.command, 'x64', False)
-
     def test_ovmf_4m(self):
         q = Qemu.QemuCommand(
             QemuEfiMachine.OVMF_Q35,
@@ -341,6 +300,9 @@ class BootToShellTest(unittest.TestCase):
         )
         self.run_cmd_check_shell(q.command)
 
+    def test_riscv64(self):
+        q = Qemu.QemuCommand(QemuEfiMachine.RISCV64)
+        self.run_cmd_check_shell(q.command)
 
 if __name__ == '__main__':
     unittest.main(verbosity=2)
